'use strict'

const utils = require('loader-utils')
const path = require('path')
const pkg = require('../package')

const parser = require('./libs/import-parser-v2')
const fs = require('fs')
const projectPath = path.resolve(__dirname, '../../../../')
const configFile = path.resolve(projectPath, 'babel.config.js')
let ignore = null
try {
  ignore = require(configFile)({ cache: () => { } }).ignore
} catch (error) {
  ignore = []
}


function _path() {
  const data = require("path");

  _path = function () {
    return data;
  };

  return data;
}

const sep = `\\${_path().sep}`;
const endSep = `(?:${sep}|$)`;
const substitution = `[^${sep}]+`;
const starPat = `(?:${substitution}${sep})`;
const starPatLast = `(?:${substitution}${endSep})`;
const starStarPat = `${starPat}*?`;
const starStarPatLast = `${starPat}*?${starPatLast}?`;

function escapeRegExp(string) {
  return string.replace(/[|\\{}()[\]^$+*?.]/g, "\\$&");
}

function pathToPattern(pattern, dirname) {
  const parts = _path().resolve(dirname, pattern).split(_path().sep);

  return new RegExp(["^", ...parts.map((part, i) => {
    const last = i === parts.length - 1;
    if (part === "**") return last ? starStarPatLast : starStarPat;
    if (part === "*") return last ? starPatLast : starPat;

    if (part.indexOf("*.") === 0) {
      return substitution + escapeRegExp(part.slice(1)) + (last ? endSep : sep);
    }

    return escapeRegExp(part) + (last ? endSep : sep);
  })].join(""));
}
const ignoreList = ignore.map(i => pathToPattern(i, projectPath))
module.exports = function (source, sourceMap) {
  const callback = this.async()
  if (!/(\.jsx?|\.tsx?|\.vue)$/.test(this.resourcePath) || /node_modules/.test(this.resourcePath) || ignoreList.some(i => i.test(this.resourcePath))) {
    callback(null, source, sourceMap)
    return
  }

  this.cacheable()
  const _this = this
  const vuxConfig = this.vux || utils.getLoaderConfig(this, 'vux')

  // 打包时无法获取选项，默认为使用 vux ui
  // if (/}\s+from(.*?)('|")vux/.test(source)) {

  //   const maps = this.vuxMaps || utils.getLoaderConfig(this, 'vuxMaps')
  // source = parser(source, function (opts) {
  //   let str = ''
  //   opts.components.forEach(function (component) {
  //     let file = `vux/${maps[component.originalName]}`
  //     if (vuxConfig.options.vuxDev) {
  //       if (vuxConfig.options.resolveVuxDir) {
  //         file = file.replace('vux/src/', vuxConfig.options.resolveVuxDir)
  //       } else {
  //         file = file.replace('vux/src/', './')
  //       }
  //     }
  //     str += `import ${component.newName} from '${file}'\n`
  //   })
  //   return str
  // }, 'vux')    
  // }

  const { code, map } = parser(source, this.resourcePath, sourceMap)
  if (map) {
    if (!map.sourcesContent || !map.sourcesContent.length) {
      map.sourcesContent = [code]
    }
    sourceMap = map
  }
  source = code
  if (/** vuxConfig.options.vuxDev && **/ /main\.js/.test(this.resourcePath)) {
    source = source.replace(/!vux\/src/g, '!.')
  }


  /**
  if (/main\.js/.test(this.resourcePath) && process.env.NODE_ENV === 'development') {
    if (this.options && this.options.context) {
      const pkgPath = vuxConfig.options.vuxDev ? path.join(this.options.context, 'package.json') : path.join(this.options.context, 'node_modules/vux/package.json')
      const vuxPkg = require(pkgPath)
      const webpackPath = path.join(this.options.context, 'node_modules/webpack/package.json')
      const webpackPkg = require(webpackPath)
      const nodeVersion = process.version.match(/^v(\d+\.\d+)/)[1]
      const style = 'background: #35495e; color: yellow;'
      if (typeof vuxConfig.options.showVuxVersionInfo === 'undefined' || vuxConfig.options.showVuxVersionInfo === true) {
        source += `\n;console.info('[VUX] %cvux@${vuxPkg.version}, vux-loader@${pkg.version}, webpack@${webpackPkg.version}, node@${nodeVersion}\\n%c[VUX] 建议反馈请访问 https://github.com/airyland/vux/issues \\n[VUX] 关闭该提示请在 vux-loader 配置  options: { showVuxVersionInfo: false }', '${style}', '')`
      }
    }
  }
  **/

  callback(null, source, sourceMap)
}